# Copyright (c) 2012 Martin Garcia <newluxfero@gmail.com>
#
# This file is part of python-producteev, and is made available under
# MIT license. See LICENSE for the full details.
from utils import unescape
import re


class Dashboard(object):
    """
    Dashboard represents an dashboard entity in Producteev.
    """

    class __raw:
            pass

    def __init__(self, api, values):
        self.__api = api
        self.__reload(values)

    def __reload(self, values):
        self.__raw.__dict__.update(values)
        self.id = int(self.__raw.id_dashboard)

    def __str__(self):
        return self.title

    def __cmp__(self, other):
        return self.id - other.id

    def __title():
        def fget(self):
            return unescape(self.__raw.title)

        def fset(self, value):
            dashboard = self.__api.call('dashboards/set_title',
                                        id_dashboard=self.id,
                                        title=value)['dashboard']
            self.__reload(dashboard)

        return locals()
    title = property(**__title())

    def __access():
        def fget(self):
            from users import User
            users = self.__api.call('dashboards/access',
                                    id_dashboard=self.id)['accesslist']
            return [User(self.__api, x['user']) for x in users]

        def fset(self):
            pass

        return locals()
    users = property(**__access())

    def __tasks():
        def fget(self):
            from tasks import Task
            tasks = self.__api.call('dashboards/tasks',
                                    id_dashboard=self.id)['tasks']
            return [Task(self.__api, x['task']) for x in tasks]

        def fset(self):
            pass

        return locals()
    tasks = property(**__tasks())

    def leave(self):
        resp = self.__api.call('dashboards/leave',
                                id_dashboard=self.id)['stats']['result']
        return resp == 'TRUE'

    def enable_smart_labels(self):
        dashboard = self.__api.call('dashboards/leave',
                                    id_dashboard=self.id, on=1)['dashboard']
        return Dashboard(self.__api, dashboard)

    def disable_smart_labels(self):
        dashboard = self.__api.call('dashboards/leave',
                                    id_dashboard=self.id, on=0)['dashboard']
        return Dashboard(self.__api, dashboard)

    def smart_labels(self, value):
        if value is True:
            self.enable_smart_labels()
        else:
            self.disable_smart_labels()

    def delete(self):
        resp = self.__api.call('dashboards/delete',
                                id_dashboard=self.id)['stats']['result']
        return resp == 'TRUE'

    def invite(self, value, message=None):
        from users import User
        kwargs = {}
        if isinstance(value, User):
            kwargs['id_user_to'] = value
            if message:
                kwargs['message'] = message
            dashboard = self.__api.call('dashboards/invite_user_by_id',
                                        **kwargs)['dashboard']
        elif isinstance(value, int):
            kwargs['id_user_to'] = value
            if message:
                kwargs['message'] = message
            dashboard = self.__api.call('dashboards/invite_user_by_id',
                                        **kwargs)['dashboard']
        elif re.match("^.+\\@(\\[?)[a-zA-Z0-9\\-\\.]+\\.([a-zA-Z]{2,3}|[0-9]{1,3})(\\]?)$", value) != None:
            kwargs['email'] = value
            if message:
                kwargs['message'] = message
            dashboard = self.__api.call('dashboards/invite_user_by_id',
                                        **kwargs)['dashboard']
        return Dashboard(self.__api, dashboard)

    def needs_upgrade(self):
        """
        Get every dashboard that needs to be upgraded.
        """
        return self.__api.call('dashboards/needs_upgrade',
                                     id_dashboard=self.id)


class Dashboards(object):
    """
    Dashboards give an interface for manage dashboards in Producteev.
    """

    def __init__(self, api):
        self.__api = api

    def new(self, title):
        return Dashboard(self.__api, self.__api.call('dashboards/create',
                                title=title)['dashboard'])

    def get(self, value):
        if isinstance(value, int):
            pass
        elif isinstance(value, Dashboard):
            value = value.id
        elif isinstance(value, str):
            try:
                value = int(str)
            except:
                # TODO: raise error
                return None
        else:
            # TODO: raise error
            return None

        dashboard = Dashboard(self.__api,
                                self.__api.call('dashboards/view',
                                id_dashboard=value)['dashboard'])
        return dashboard

    def __list():
        def fget(self):
            dashboards = self.__api.call('dashboards/show_list')['dashboards']
            return [Dashboard(self.__api, x['dashboard']) for x in dashboards]

        def fset(self):
            pass

        return locals()
    list = property(**__list())

    def confirm(self, id_dashboard):
        """
        Confirm invitation for a dashboard.
        """
        if isinstance(id_dashboard, int):
            pass
        elif isinstance(id_dashboard, Dashboard):
            id_dashboard = id_dashboard.id
        elif isinstance(id_dashboard, str):
            try:
                id_dashboard = int(id_dashboard)
            except:
                # TODO: Raise error
                return None
        else:
            # TODO: Raise error
            return None

        dashboards = self.__api.call('dashboards/confirm',
                                     id_dashboard=id_dashboard)['dashboards']
        return [Dashboard(self.__api, x['dashboard']) for x in dashboards]

    def refuse(self, id_dashboard):
        """
        Refuse invitation for a dashboard.
        """
        if isinstance(id_dashboard, int):
            pass
        elif isinstance(id_dashboard, Dashboard):
            id_dashboard = id_dashboard.id
        elif isinstance(id_dashboard, str):
            try:
                id_dashboard = int(id_dashboard)
            except:
                # TODO: Raise error
                return None
        else:
            # TODO: Raise error
            return None

        resp = self.__api.call('dashboards/refuse',
                                id_dashboard=id_dashboard)['stats']['result']
        return resp == 'TRUE'

    def need_upgrade(self):
        """
        Get every dashboard that needs to be upgraded.
        """
        dashboards = self.__api.call('dashboards/need_upgrade_list')['dashboards']
        return [Dashboard(self.__api, x['dashboard']) for x in dashboards]

    def reorder(self, dashboard_list):
        """
        Reorder dashboard position.
        """
        if not isinstance(dashboard_list, list):
            # TODO: raise error
            return None

        dash_list = []
        for dash in dashboard_list:
            if isinstance(dash, int):
                dash_list.append(dash)
            elif isinstance(dash, Dashboard):
                dash_list.append(dash.id)
            elif isinstance(dash, str):
                try:
                    dash = int(dash)
                    dash_list.append(dash)
                except:
                    pass
            else:
                continue

        dashboards = self.__api.call('dashboards/reorder',
                                     id_dashboard=dash_list)['dashboards']
        return [Dashboard(self.__api, x['dashboard']) for x in dashboards]



"Producteev API library for Python."
# Copyright (c) 2012 Martin Garcia <newluxfero@gmail.com>
#
# This file is part of python-producteev, and is made available under
# MIT license. See LICENSE for the full details.

from .core import Producteev, AuthError
from .timezones import Zones as TimeZones

VERSION = (0, 1, 4)

__author__ = "Martin Garcia"
__contact__ = "newluxfero@gmail.com"
__homepage__ = "http://github.com/magarcia/python-producteev"
__version__ = '.'.join([str(i) for i in VERSION])
__release__ = 'beta'



# Copyright (c) 2012 Martin Garcia <newluxfero@gmail.com>
#
# This file is part of python-producteev, and is made available under
# MIT license. See LICENSE for the full details.
from utils import unescape


class Subtask(object):
    """
    Subtask represents an subtask entity in Producteev.
    """

    class __raw:
            pass

    def __init__(self, api, values):
        self.__api = api
        self.__reload(values)

    def __reload(self, values):
        self.__raw.__dict__.update(values)
        self.id = int(self.__raw.id_subtask)

    def __str__(self):
        return self.title

    def __cmp__(self, other):
        return self.id - other.id

    def __title():
        def fget(self):
            return unescape(self.__raw.title)

        def fset(self, value):
            subtask = self.__api.call('subtasks/set_title', id_subtask=self.id,
                                   title=value)['subtask']
            self.__reload(subtask)

        return locals()
    title = property(**__title())

    def __position():
        def fget(self):
            return self.__raw.position

        def fset(self, value):
            subtask = self.__api.call('subtasks/set_position',
                                      id_subtask=self.id)['subtask']
            self.__reload(subtask)

        return locals()
    position = property(**__position())

    def delete(self):
        resp = self.__api.call('subtasks/delete',
                               id_subtask=self.id)['stats']['result']
        return resp == 'TRUE'


class Subtasks(object):
    """
    Subtasks give an interface for manage subtasks in Producteev.
    """

    def __init__(self, api):
        self.__api = api

    def new(self, task, title):
        from tasks import Task
        if isinstance(task, int):
            pass
        elif isinstance(task, Task):
            task = task.id
        elif isinstance(task, str):
            try:
                task = int(task)
            except:
                # TODO: raise error
                return None
        else:
            # TODO: raise error
            return None

        subtask = self.__api.call('subtasks/create', id_task=task,
                                   title=title)['subtask']

        return Subtask(self.__api, subtask)



# Copyright (c) 2012 Martin Garcia <newluxfero@gmail.com>
#
# This file is part of python-producteev, and is made available under
# MIT license. See LICENSE for the full details.
from utils import unescape

STATUS = ('UNDONE', 'DONE')


class Task(object):
    """
    Task represents an task entity in Producteev.
    """

    class __raw:
            pass

    def __init__(self, api, values):
        self.__api = api
        self.__reload(values)

    def __reload(self, values):
        self.__raw.__dict__.update(values)
        self.id = int(self.__raw.id_task)

    def __str__(self):
        return self.title

    def __cmp__(self, other):
        return self.id - other.id

    def __set_title():
        def fget(self):
            return unescape(self.__raw.title)

        def fset(self, value):
            task = self.__api.call('tasks/set_title', id_task=self.id,
                                   title=value)['task']
            self.__reload(task)

        return locals()
    title = property(**__set_title())

    def __set_status():
        def fget(self):
            return STATUS[int(self.__raw.status) - 1]

        def fset(self, value):
            if isinstance(value, int):
                task = self.__api.call('tasks/set_status',
                                        id_task=self.id,
                                        status=value)['task']
            elif isinstance(value, str):
                try:
                    value = int(value)
                    task = self.__api.call('tasks/set_status',
                                            id_task=self.id,
                                            status=value)['task']
                except:
                    try:
                        value = STATUS.index(value.upper()) + 1
                        task = self.__api.call('tasks/set_status',
                                                id_task=self.id,
                                                status=value)['task']
                    except:
                        task = None
            if not task:
                self.__reload(task)
        return locals()
    status = property(**__set_status())

    def __set_star():
        def fget(self):
            return self.__raw.star

        def fset(self, value):
            task = self.__api.call('tasks/set_star', id_task=self.id,
                                    star=value)['task']
            self.__reload(task)
        return locals()
    star = property(**__set_star())

    def __responsible():
        def fget(self):
            from users import User
            return User(self.__api, self.__raw.id_responsible)

        def fset(self, value):
            from users import User
            if isinstance(value, int):
                task = self.__api.call('tasks/set_responsible',
                                       id_task=self.id,
                                       id_responsible=value)['task']
            elif isinstance(value, User):
                task = self.__api.call('tasks/set_responsible',
                                       id_task=self.id,
                                       id_responsible=value.id)['task']
            elif value is None:
                task = self.__api.call('tasks/unset_responsible',
                                       id_task=self.id)['task']
            self.__reload(task)
        return locals()
    responsible = property(**__responsible())

    def __set_deadline():
        def fget(self):
            if self.__raw.deadline == '':
                return None
            else:
                from datetime import datetime
                from core import PRODUCTEEV_DATE_FORMAT
                return datetime.strptime(
                    ' '.join(self.__raw.deadline.split(' ')[0:-1]),
                    PRODUCTEEV_DATE_FORMAT)

        def fset(self, value):
            if value:
                task = self.__api.call('tasks/set_deadline',
                                       id_task=self.id,
                                       deadline=value)['task']
            else:
                task = self.__api.call('tasks/unset_deadline',
                                       id_task=self.id)['task']
            self.__reload(task)
        return locals()
    deadline = property(**__set_deadline())

    def __set_reminder():
        def fget(self):
            return self.__raw.reminder

        def fset(self, value):
            if value:
                task = self.__api.call('tasks/set_reminder',
                                        id_task=self.id, reminder=value)['task']
            else:
                task = self.__api.call('tasks/set_reminder',
                                        id_task=self.id, reminder=0)['task']
            self.__reload(task)

        return locals()
    reminder = property(**__set_reminder())

    def __set_repeating():
        def fget(self):
            return self.__raw.repeating_interval, self.__raw.repeating_value

        def fset(self, value, interval=1):
            if value:
                task = self.__api.call('tasks/set_repeating',
                                       id_task=self.id,
                                       repeating_interval=interval,
                                       repeating_value=value)['task']
            else:
                task = self.__api.call('tasks/unset_repeating',
                                       id_task=self.id)['task']
            self.__reload(task)
        return locals()
    repeating = property(**__set_repeating())

    def delete(self):
        resp = self.__api.call('tasks/delete',
                               id_task=self.id)['stats']['result']
        return resp == 'TRUE'

    def __set_labels():
        def fget(self):
            from labels import Label
            return [Label(self.__api, x['label']) for x in self.__raw.labels]

        def fset(self, value):
            # TODO: tasks/change_labels
            pass
        return locals()
    labels = property(**__set_labels())

    def __set_workspace():
        def fget(self):
            return self.__api(int(self.__raw.id_dashboard))

        def fset(self, value):
            from dashboards import Dashboard
            if isinstance(value, int):
                task = self.__api.call('tasks/set_workspace',
                                        id_task=self.id,
                                        id_dashboard=value)['task']
            elif isinstance(value, Dashboard):
                task = self.__api.call('tasks/set_workspace',
                                        id_task=self.id,
                                        id_dashboard=value.id)['task']
            else:
                task = None
            if not task:
                self.__reload(task)
        return locals()
    dashboard = property(**__set_workspace())

    def __notes():
        """
        Get every note of a task.
        """
        def fget(self):
            task = self.__api.call('tasks/notes_get', id_task=self.id)
            return task

        def fset(self):
            pass
        return locals()
    notes = property(**__notes())

    def new_note(self, message):
        """
        Create a note attached to a task.
        """
        return self.__api.call('tasks/note_create', id_task=self.id,
                                message=message)

    def __activities():
        """
        Get every note of a task.
        """
        def fget(self):
            from activities import Activity
            activities = self.__api.call('tasks/activities_get',
                                         id_task=self.id)['activities']
            return [Activity(self.__api, x['activity']) for x in activities]

        def fset(self):
            pass
        return locals()
    activities = property(**__activities())

    def __get_subtasks():
        def fget(self):
            from subtasks import Subtask
            subtasks = self.__api.call('tasks/subtasks',
                                        id_task=self.id)['subtasks']
            return [Subtask(self.__api, x) for x in subtasks]

        def fset(self, value):
            pass
        return locals()
    subtasks = property(**__get_subtasks())


class Tasks(object):
    """
    Tasks give an interface for manage tasks in Producteev.
    """

    def __init__(self, api):
        self.__api = api

    def new(self, title, **kwargs):
        """
        """
        return Task(self.__api, self.__api.call('tasks/create', title=title,
                                   **kwargs)['task'])

    def get(self, id_task, **kwargs):
        """
        """
        if isinstance(id_task, int):
            pass
        elif isinstance(id_task, Task):
            id_task = id_task.id
        elif isinstance(id_task, str):
            try:
                id_task = int(id_task)
            except:
                # TODO: raise error
                return None
        else:
            # TODO: raise error
            return None

        return Task(self.__api, self.__api.call('tasks/view', id_task=id_task,
                             **kwargs)['task'])

    def list(self, **kwargs):
        """
        """
        tasks = self.__api.call('tasks/my_tasks', **kwargs)['tasks']
        return [Task(self.__api, x['task']) for x in tasks]

    def list_all(self, **kwargs):
        """
        """
        tasks = self.__api.call('tasks/show_list', **kwargs)['tasks']
        return [Task(self.__api, x['task']) for x in tasks]

    def archived(self, **kwargs):
        tasks = self.__api.call('tasks/archived', **kwargs)['tasks']
        return [Task(self.__api, x['task']) for x in tasks]

    def list_team(self, **kwargs):
        tasks = self.__api.call('tasks/my_team_tasks', **kwargs)['tasks']
        return [Task(self.__api, x['task']) for x in tasks]

    def get_note(self, id_note):
        """
        Get a note.
        """
        return self.__api.call('tasks/note_view', id_note=id_note)

    def delete_note(self, id_note):
        """
        Deleta a note.
        """
        resp = self.__api.call('tasks/note_delete',
                                id_note=id_note)['stats']['result']
        return resp == 'TRUE'



# -*- coding: utf-8 -*-

# Copyright (c) 2012 Martin Garcia <newluxfero@gmail.com>
#
# This file is part of python-producteev, and is made available under
# MIT license. See LICENSE for the full details.
from utils import unescape

LANG_ID = ('English', 'Français', 'Italiano', 'Deutsch')
SORT_BY = ('deadline', 'manager', 'labels', 'time_lastchange',
           'workspace', 'priority', 'title', 'time_created')


class User(object):
    """
    User represents an user entity in Producteev.
    """

    class __raw:
            pass

    def __init__(self, api, id=None):
        self.__api = api
        self.__get_user_info(id)

    def __get_user_info(self, id):
        if isinstance(id, int):
            values = self.__api.call('users/view', id_colleague=id)['user']
        else:
            values = self.__api.call('users/view')['user']
        self.__reload(values)

    def __cmp__(self, other):
        return self.id - other.id

    def __reload(self, values):
        self.__raw.__dict__.update(values)
        self.firstname = unescape(self.__raw.firstname)
        self.lastname = unescape(self.__raw.lastname)
        self.company = unescape(self.__raw.company)
        self.colleagues = self.__raw.colleagues
        self.id = int(self.__raw.id_user)
        self.lang = LANG_ID[int(self.__raw.lang) + 1]

    def __full_name():
        def fget(self):
            return '%s %s' % (self.firstname, self.lastname)

        def fset(self, value):
            full_name = value.split(' ')
            self.firstname = full_name.pop(0)
            self.lastname = ' '.joint(full_name)

        return locals()
    full_name = property(**__full_name())

    def __default_dashboard():
        def fget(self):
            return self.__api.dashboards.get(int(self.__raw.default_dashboard))

        def fset(self, value):
            from dashboards import Dashboard

            if isinstance(value, int):
                pass
            elif isinstance(value, Dashboard):
                value = value.id
            elif isinstance(value, str):
                try:
                    value = int(value)
                except:
                    # TODO: raise error
                    value = None
            else:
                # TODO: raise error
                value = None

            if not value is None:
                user = self.__api.call('users/set_default_dashboard',
                                        id_dashboard=value)['user']
                self.__reload(user)

        return locals()
    default_dashboard = property(**__default_dashboard())

    def __sort_by():
        def fget(self):
            return SORT_BY[int(self.__raw.sort_by) - 1]

        def fset(self, value):
            if isinstance(value, int):
                value += 1
            elif isinstance(value, str):
                try:
                    value = int(value) + 1
                except:
                    try:
                        value = SORT_BY.index(value.lower()) + 1
                    except:
                        # TODO: raise error
                        value = None
            else:
                # TODO: raise error
                value = None

            if not value is None:
                user = self.__api.call('users/set_sort_by',
                                       sort=value)['user']
                self.__reload(user)

        return locals()
    sort_by = property(**__sort_by())

    def __timezone():
        def fget(self):
            return self.__raw.timezone

        def fset(self, value):
            user = self.__api.call('users/set_timezone',
                                   timezone=value)['user']
            self.__reload(user)

        return locals()
    timezone = property(**__timezone())


class Users(object):
    """
    Users give an interface for manage users in Producteev.
    """

    def __init__(self, api):
        self.__api = api
        self.me = User(self.__api)

    def __list():
        def fget(self):
            return self.me.colleagues

        def fset(self):
            pass

        return locals()
    list = property(**__list())

    def get(self, id):
        if isinstance(id, int):
            pass
        elif isinstance(id, User):
            id = id.id
        elif isinstance(id, str):
            try:
                id = int(id)
            except:
                # TODO: raise error
                return None
        else:
            # TODO: raise error
            return None
        return User(self.__api, id)



# Copyright (c) 2012 Martin Garcia <newluxfero@gmail.com>
#
# This file is part of python-producteev, and is made available under
# MIT license. See LICENSE for the full details.

Zones = {'Europe': ['Dublin', 'Lisbon', 'London', 'Amsterdam', 'Andorra', 'Belgrade', 'Berlin', 'Brussels', 'Budapest', 'Copenhagen', 'Gibraltar', 'Luxembourg', 'Madrid', 'Malta', 'Monaco', 'Oslo', 'Paris', 'Prague', 'Rome', 'Stockholm', 'Tirane', 'Vaduz', 'Vienna', 'Warsaw', 'Zurich', 'Athens', 'Bucharest', 'Chisinau', 'Helsinki', 'Istanbul', 'Kaliningrad', 'Kiev', 'Minsk', 'Riga', 'Sofia', 'Tallinn', 'Vilnius', 'Moscow', 'Samara'], 'Asia': ['Amman', 'Beirut', 'Damascus', 'Gaza', 'Jerusalem', 'Nicosia', 'Aden', 'Baghdad', 'Bahrain', 'Kuwait', 'Qatar', 'Riyadh', 'Tehran', 'Baku', 'Dubai', 'Muscat', 'Tbilisi', 'Yerevan', 'Kabul', 'Aqtau', 'Aqtobe', 'Ashgabat', 'Dushanbe', 'Karachi', 'Tashkent', 'Yekaterinburg', 'Calcutta', 'Colombo', 'Katmandu', 'Almaty', 'Bishkek', 'Dhaka', 'Omsk', 'Thimphu', 'Rangoon', 'Bangkok', 'Hovd', 'Jakarta', 'Krasnoyarsk', 'Phnom_Penh', 'Saigon', 'Vientiane', 'Brunei', 'Choibalsan', 'Hong_Kong', 'Irkutsk', 'Kuala_Lumpur', 'Macau', 'Makassar', 'Manila', 'Shanghai', 'Singapore', 'Taipei', 'Ulaanbaatar', 'Dili', 'Jayapura', 'Pyongyang', 'Seoul', 'Tokyo', 'Yakutsk', 'Vladivostok', 'Kamchatka', 'Magadan'], 'Australia': ['Perth', 'Adelaide', 'Darwin', 'Brisbane', 'Hobart', 'Sydney'], 'Africa': ['Abidjan', 'Accra', 'Bamako', 'Banjul', 'Bissau', 'Casablanca', 'Conakry', 'Dakar', 'El_Aaiun', 'Freetown', 'Lome', 'Monrovia', 'Nouakchott', 'Ouagadougou', 'Sao_Tome', 'Algiers', 'Bangui', 'Brazzaville', 'Ceuta', 'Douala', 'Kinshasa', 'Lagos', 'Libreville', 'Luanda', 'Malabo', 'Ndjamena', 'Niamey', 'Porto-Novo', 'Tunis', 'Windhoek', 'Blantyre', 'Bujumbura', 'Cairo', 'Gaborone', 'Harare', 'Johannesburg', 'Kigali', 'Lubumbashi', 'Lusaka', 'Maputo', 'Maseru', 'Mbabane', 'Tripoli', 'Addis_Ababa', 'Asmara', 'Dar_es_Salaam', 'Djibouti', 'Kampala', 'Khartoum', 'Mogadishu', 'Nairobi'], 'US': ['Eastern'], 'Etc': ['GMT'], 'Antarctica': ['Palmer', 'Rothera', 'Syowa', 'Mawson', 'Vostok', 'Davis', 'Casey', 'DumontDUrville'], 'Indian': ['Antananarivo', 'Comoro', 'Mayotte', 'Mahe', 'Mauritius', 'Reunion', 'Kerguelen', 'Maldives', 'Chagos', 'Cocos', 'Christmas'], 'Atlantic': ['Bermuda', 'Stanley', 'South_Georgia', 'Azores', 'Cape_Verde', 'Canary', 'Faroe', 'Reykjavik', 'St_Helena'], 'Pacific': ['Apia', 'Midway', 'Niue', 'Pago_Pago', 'Fakaofo', 'Honolulu', 'Johnston', 'Rarotonga', 'Tahiti', 'Marquesas', 'Gambier', 'Pitcairn', 'Easter', 'Galapagos', 'Palau', 'Guam', 'Port_Moresby', 'Saipan', 'Truk', 'Efate', 'Guadalcanal', 'Kosrae', 'Noumea', 'Ponape', 'Norfolk', 'Auckland', 'Fiji', 'Funafuti', 'Kwajalein', 'Majuro', 'Nauru', 'Tarawa', 'Wake', 'Wallis', 'Enderbury', 'Tongatapu', 'Kiritimati'], 'America': ['Anchorage', 'Los_Angeles', 'Tijuana', 'Vancouver', 'Whitehorse', 'Dawson_Creek', 'Denver', 'Edmonton', 'Hermosillo', 'Mazatlan', 'Phoenix', 'Yellowknife', 'Belize', 'Chicago', 'Costa_Rica', 'El_Salvador', 'Guatemala', 'Managua', 'Mexico_City', 'Regina', 'Tegucigalpa', 'Winnipeg', 'Bogota', 'Cayman', 'Grand_Turk', 'Guayaquil', 'Havana', 'Iqaluit', 'Jamaica', 'Lima', 'Montreal', 'Nassau', 'New_York', 'Panama', 'Port-au-Prince', 'Toronto', 'Caracas', 'Anguilla', 'Antigua', 'Aruba', 'Asuncion', 'Barbados', 'Boa_Vista', 'Campo_Grande', 'Cuiaba', 'Curacao', 'Dominica', 'Grenada', 'Guadeloupe', 'Guyana', 'Halifax', 'La_Paz', 'Manaus', 'Martinique', 'Montserrat', 'Port_of_Spain', 'Porto_Velho', 'Puerto_Rico', 'Rio_Branco', 'Santiago', 'Santo_Domingo', 'St_Kitts', 'St_Lucia', 'St_Thomas', 'St_Vincent', 'Thule', 'Tortola', 'St_Johns', 'Araguaina', 'Argentina', 'Bahia', 'Belem', 'Cayenne', 'Fortaleza', 'Godthab', 'Maceio', 'Miquelon', 'Montevideo', 'Paramaribo', 'Recife', 'Sao_Paulo', 'Noronha', 'Scoresbysund', 'Danmarkshavn']}


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
import producteev

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


if sys.argv[1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()

if sys.argv[1] == 'docs':
    try:
        import sphinx
    except ImportError:
        sys.stderr.write('You must be install sphinx for make docs.\n')
        sys.stderr.write('\t pip install sphinx\n')
        sys.exit(-1)

    build_command = 'make html'
    if len(sys.argv) >= 3:
        if sys.argv[2].lower() == 'latex':
            build_command = 'make latex'
        elif sys.argv[2].lower() == 'pdf':
            build_command = 'make latexpdf'
        elif sys.argv[2].lower() == 'html':
            build_command = 'make html'
        else:
            sys.stderr.write('Unrecognized format.\n')
            sys.exit(-1)

    docs_dir = os.path.join(os.path.dirname(__file__), 'docs')
    os.chdir(docs_dir)
    os.system(build_command)
    os.chdir(os.path.pardir)
    sys.exit()

required = []


def read(fname):
    try:
        return open(os.path.join(os.path.dirname(__file__), fname)).read()
    except:
        return "This is a Python wrapper for the Producteev API."


setup(
    name='producteev',
    version=producteev.__version__,
    description='Producteev API.',
    author='Martín García',
    author_email='newluxfero@gmail.com',
    long_description = read('README.rst'),
    url='https://github.com/magarcia/python-producteev',
    packages=['producteev'],
    package_data={'': ['LICENSE',]},
    include_package_data=True,
    install_requires=required,
    license='MIT',
    test_suite='tests',
    classifiers=(
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        # 'Programming Language :: Python :: 3.0',
        # 'Programming Language :: Python :: 3.1',
    ),
)


# Copyright (c) 2012 Martin Garcia <newluxfero@gmail.com>
#
# This file is part of python-producteev, and is made available under
# MIT license. See LICENSE for the full details.
from utils import unescape


class Label(object):
    """
    Label represents an label entity in Producteev.
    """

    class __raw:
            pass

    def __init__(self, api, values):
        self.__api = api
        self.__reload(values)

    def __reload(self, values):
        self.__raw.__dict__.update(values)
        self.id = int(self.__raw.id_label)

    def __str__(self):
        return self.title

    def __cmp__(self, other):
        return self.id - other.id

    def __set_title():
        def fget(self):
            return unescape(self.__raw.title)

        def fset(self, value):
            label = self.__api.call('labels/set_title', id_label=self.id,
                                   title=value)['label']
            self.__reload(label)

        return locals()
    title = property(**__set_title())

    def __tasks():
        def fget(self):
            from tasks import Task
            return [Task(self.__api, x) for x in self.__raw.tasks]

        def fset(self, value):
            pass

        return locals()
    tasks = property(**__tasks())

    def delete(self):
        resp = self.__api.call('labels/delete',
                                id_label=self.id)['stats']['result']
        return resp == 'TRUE'


class Labels(object):
    """
    Labels give an interface for manage labels in Producteev.
    """

    def __init__(self, api):
        self.__api = api

    def new(self, title, id_dashboard):
        return Label(self.__api, self.__api.call('labels/create',
                            title=title, id_dashboard=id_dashboard)['label'])

    def get(self, id_label):
        if isinstance(id_label, int):
            pass
        elif isinstance(id_label, Label):
            id_label = id_label.id
        elif isinstance(id_label, str):
            try:
                id_label = int(id_label)
            except:
                # TODO: raise error
                return None
        else:
            # TODO: raise error
            return None
        return Label(self.__api, self.__api.call('labels/view',
                        id_label=id_label)['label'])

    def list(self, **kwargs):
        lables = self.__api.call('labels/show_list', **kwargs)['labels']
        return [Label(self.__api, x['label']) for x in lables]



# Copyright (c) 2012 Martin Garcia <newluxfero@gmail.com>
#
# This file is part of python-producteev, and is made available under
# MIT license. See LICENSE for the full details.


class Activity(object):
    """
    Activity represents an activity entity in Producteev.
    """

    class __raw:
            pass

    def __init__(self, api, values):
        self.__api = api
        self.__reload(values)

    def __reload(self, values):
        self.__raw.__dict__.update(values)
        self.id = int(self.__raw.id_activity)

    def __cmp__(self, other):
        return self.id - other.id

    def set_read(self):
        """
        Set activity status as read.
        """
        activities = self.__api.call('activities/set_read',
                                     id_activity=self.id)['activities']
        return [Activity(self.__api, x['activity']) for x in activities]


class Activities(object):
    """
    Activities give an interface for manage activities in Producteev.
    """

    def __init__(self, api):
        self.__api = api

    def __list():
        """
        Get activities. Don't return activities older than 1 month.
        """
        def fget(self):
            activities = self.__api.call('activities/show_activities')['activities']
            return [Activity(self.__api, x['activity']) for x in activities]

        def fset(self):
            pass

        return locals()
    list = property(**__list())

    def get(self, id_activity):
        """
        Get activity.
        """
        if isinstance(id_activity, int):
            pass
        elif isinstance(id_activity, Activity):
            id_activity = id_activity.id
        elif isinstance(id_activity, str):
            try:
                id_activity = int(id_activity)
            except:
                # TODO: raise error
                return None
        else:
            # TODO: raise error
            return None

        activity = self.__api.call('tasks/activity_view')['activity']
        return Activity(self.__api, activity)

    def __notifications():
        """
        Get every notifications (notifications are activities that current user
        is concerned about). Don't return notifications older than 1 month.
        """
        def fget(self):
            activities = self.__api.call('activities/show_notifications')['activities']
            return [Activity(self.__api, x['activity']) for x in activities]

        def fset(self):
            # TODO: raise inmutable object
            pass

        return locals()
    notifications = property(**__notifications())

    def set_notifications_read(self, id_dashboard, last_id):
        """
        Set notifications status as read.
        """
        from dashboards import Dashboard
        if isinstance(id_dashboard, int):
            pass
        elif isinstance(id_dashboard, Dashboard):
            id_dashboard = id_dashboard.id
        elif isinstance(id_dashboard, str):
            try:
                id_dashboard = int(id_dashboard)
            except:
                # TODO: raise error
                return None
        else:
            # TODO: raise error
            return None

        if isinstance(last_id, int):
            pass
        elif isinstance(last_id, Activity):
            last_id = last_id.id
        elif isinstance(last_id, str):
            try:
                last_id = int(last_id)
            except:
                # TODO: raise error
                return None
        else:
            # TODO: raise error
            return None

        activities = self.__api.call('activities/notifications_set_read',
                                     id_dashboard=id_dashboard,
                                     last_id=last_id)['activities']
        return [Activity(self.__api, x['activity']) for x in activities]

    def set_activities_read(self, activities_list):
        """
        Set activities status as read.
        """
        if not isinstance(activities_list, list):
            # TODO: raise error
            return None

        act_list = []
        for act in activities_list:
            if isinstance(act, int):
                act_list.append(act)
            elif isinstance(act, Activity):
                act_list.append(act.id)
            elif isinstance(act, str):
                try:
                    act = int(act)
                    act_list.append(act)
                except:
                    pass
            else:
                continue

        activities = self.__api.call('activities/set_read',
                                     id_activity=act_list)['activities']
        return [Activity(self.__api, x['activity']) for x in activities]



# Copyright (c) 2012 Martin Garcia <newluxfero@gmail.com>
#
# This file is part of python-producteev, and is made available under
# MIT license. See LICENSE for the full details.

from hashlib import md5
from urllib import urlencode
import urllib2
import json
from datetime import datetime

from users import Users
from activities import Activities
from labels import Labels
from tasks import Tasks
from dashboards import Dashboards
from subtasks import Subtasks


PRODUCTEEV_DATE_FORMAT = "%a, %d %b %Y %H:%M:%S"


class ProducteevError(Exception):
    """
    """
    def __init__(self, type, message, response=None):
        self.type = type
        self.message = message
        self.response = response

    def __str__(self):
        return "%s (%s)" % (self.type, self.message)

    def __repr__(self):
        return "%s(type=%s)" % (self.__class__.__name__, self.type)


class AuthError(ProducteevError):
    def __init__(self, message, response=None):
        super(AuthError, self).__init__('Authentication Error', message, response)


class Producteev():
    """
    """
    BASE_URL = "https://api.producteev.com/"

    def __init__(self, api_key, secret):
        self.api_key = api_key
        self.secret = secret
        self.user = None
        self.token = None

    def __sign_params(self, params={}):
        """
        Add a signature hash to params.
        """
        params["api_key"] = self.api_key
        plain_signature = ''.join(
            [''.join(p) for p in sorted(params.items(), key=lambda t: t[0])])
        plain_signature += self.secret
        params["api_sig"] = md5(plain_signature).hexdigest()
        return params

    def __compose_url(self, path, params=None):
        """
        Compose url from path and params.
        """
        params = self.__sign_params(params)
        return "%s%s?%s" % (self.BASE_URL, '%s.json' % path, urlencode(params))

    def __handle_response(self, response):
        """
        Load response to python dictonary.
        """
        return json.loads(response.read())

    def __request(self, path, **params):
        """
        Make request to Producteev API
        """
        for k, v in params.items():
            params[k] = str(v)
        url = self.__compose_url(path, params)
        r = urllib2.urlopen(url)
        return self.__handle_response(r)

    def call(self, path, **params):
        """
        Call to a Producteev API method, with any path of here_ and the required
        param for the method.

        Example:
            call('users/view', id_colleague=1234)

        .. _here: http://code.google.com/p/producteev-api/wiki/methodsDescriptions#Summary
        """
        if not self.token:
            raise AuthError('Token undefined, login required')
        return self.__request(path, token=self.token, **params)

    def get_time(self):
        """
        Get time from Producteev server.
        """
        time = self.call('time')['time']['value']
        return datetime.strptime(
            ' '.join(time.split(' ')[0:-1]),
            PRODUCTEEV_DATE_FORMAT)

    def login(self, email, password):
        """
        Login user to Producteev.
        """
        response = self.__request(
                        'users/login',
                        email=email,
                        password=password)
        self.token = response['login']['token']
        self.users = Users(self)
        self.tasks = Tasks(self)
        self.dashboards = Dashboards(self)
        self.labels = Labels(self)
        self.activities = Activities(self)
        self.subtasks = Subtasks(self)
